@@ -2,7 +2,7 @@ __author__ = 'Altertech, https://www.altertech.com/'
 __copyright__ = 'Copyright (C) 2019 Altertech'
 __license__ = 'MIT'
 
-__version__ = '0.4.36'
+__version__ = '0.5.2'
 
 import rapidtables
 import neotermcolor
@@ -33,6 +33,8 @@ from finac.core import account_info
 
 from finac.core import account_update
 
+from finac.core import archive_account
+
 # transaction methods
 from finac.core import transaction_create, transaction_complete
 from finac.core import transaction_move, transaction_delete
@@ -49,7 +51,7 @@ from finac.core import account_statement, account_statement_summary
 from finac.core import account_list, account_list_summary
 
 # purges
-from finac.core import purge, transaction_purge
+from finac.core import purge, transaction_purge, cleanup
 
 # caches
 from finac.core import preload, exec_query
