@@ -2,7 +2,7 @@ __author__ = 'Altertech, https://www.altertech.com/'
 __copyright__ = 'Copyright (C) 2019 Altertech'
 __license__ = 'MIT'
 
-__version__ = '0.4.36'
+__version__ = '0.5.2'
 
 from sqlalchemy.exc import IntegrityError
 from cachetools import TTLCache
@@ -43,9 +43,13 @@ ACCOUNT_METAL = 403
 
 ACCOUNT_REALITY = 500
 
+# IDs 800 - 899 reserved for custom account types
+
 # taxes
 ACCOUNT_TAX = 1000
 
+# IDs 1800 - 1899 reserved for custom account types
+
 # special
 ACCOUNT_TRANSIT = 2000
 ACCOUNT_ESCROW = 2001
@@ -203,8 +207,8 @@ def core_method(f):
             if 'error' in result:
                 raise _exceptions.get(result['error']['code'], RuntimeError)(
                     result['error'].get('message'))
-            logging.debug('API response {} {}'.format(result['id'],
-                                                      result['result']))
+            logger.debug('API response {} {}'.format(result['id'],
+                                                     result['result']))
             return result['result']
 
     return do
@@ -264,31 +268,6 @@ def exec_query(q, _time_ms=False):
     """
     Execute FinacQL query statement
 
-    Currenly only function call statements are supported:
-
-        SELECT <function>([args, kwargs])
-
-        e.g.
-
-        SELECT account_balance("myaccount")
-
-    Supported core functions:
-
-    * get_version
-    * asset_list
-    * asset_list_rates
-    * asset_rate^
-    * asset_rate_range^
-    * account_info
-    * account_statement
-    * account_list
-    * account_balance^
-    * account_balance_range^
-
-    Functions marked with "^" support data column assignment with "AS":
-
-        SELECT account_balance("myaccount") AS myacc
-
     Args:
         q: query to execute
 
@@ -581,6 +560,7 @@ def init(db=None, **kwargs):
         redis_db: Redis database (default: 0)
         redis_timeout: Redis server timeout
         redis_blocking_timeout: Redis lock acquisition timeout
+        custom_account_types: custom account types dict
 
     Note: if Redis server is specified, Finac will use it for integrity locking
           (if enabled). In this case, lock tokens become Redis lock objects.
@@ -592,6 +572,35 @@ def init(db=None, **kwargs):
             rate_cache_ttl = v
         elif k == 'rate_cache_size':
             rate_cache_size = v
+        elif k == 'custom_account_types':
+            for act in v:
+                try:
+                    account_name = act['name']
+                except KeyError:
+                    raise RuntimeError('Custom account name is not specified')
+                if account_name in ACCOUNT_TYPE_NAMES:
+                    raise RuntimeError(
+                        'Account type {} is already registered'.format(
+                            account_name))
+                try:
+                    code = act['code']
+                except KeyError:
+                    raise RuntimeError(
+                        'Account code is not specified for {}'.format(
+                            account_name))
+                if code in ACCOUNT_TYPE_IDS:
+                    raise RuntimeError(
+                        'Account code {} is already in use'.format(code))
+                elif (code < 800 or code > 899) and (code < 1800 or
+                                                     code > 1899):
+                    raise RuntimeError(
+                        ('Account code {} is invalid: should '
+                         'be in ranges 800..899 or 1800..1899').format(code))
+                else:
+                    ACCOUNT_TYPE_NAMES[code] = account_name
+                    ACCOUNT_TYPE_IDS[account_name] = code
+                    if act.get('passive'):
+                        PASSIVE_ACCOUNTS.append(code)
         elif not hasattr(config, k):
             raise RuntimeError('Parameter {} is invalid'.format(k))
         setattr(config, k, v)
@@ -801,7 +810,7 @@ def asset_set_rate(asset_from, asset_to=None, value=None, date=None):
         date = parse_date(date, return_timestamp=False)
     if asset_from.find('/') != -1 and asset_to is None:
         asset_from, asset_to = asset_from.split('/')
-    logging.info('Setting rate for {}/{} to {} for {}'.format(
+    logger.info('Setting rate for {}/{} to {} for {}'.format(
         asset_from.upper(), asset_to.upper(), value, format_date(date)))
     get_db().execute(sql("""
     insert into asset_rate (asset_from_id, asset_to_id, d, value)
@@ -830,8 +839,9 @@ def asset_delete_rate(asset_from, asset_to=None, date=None):
     if asset_from.find('/') != -1 and asset_to is None:
         asset_from, asset_to = asset_from.split('/')
     date = parse_date(date, return_timestamp=False)
-    logging.info('Deleting rate for {}/{} for {}'.format(
-        asset_from.upper(), asset_to.upper(), format_date(date)))
+    logger.info('Deleting rate for {}/{} for {}'.format(asset_from.upper(),
+                                                        asset_to.upper(),
+                                                        format_date(date)))
     if not get_db().execute(sql("""
     delete from asset_rate where
         asset_from_id=(select id from asset where code=:f)
@@ -1635,7 +1645,7 @@ def transaction_complete(transaction_ids,
         transaction_ids: single or list/tuple of transaction ID
         completion_date: completion date (default: now)
     """
-    logging.info('Completing transaction {}'.format(transaction_ids))
+    logger.info('Completing transaction {}'.format(transaction_ids))
     if completion_date is None:
         completion_date = parse_date(return_timestamp=False)
     if config.keep_integrity:
@@ -1662,7 +1672,7 @@ def transaction_complete(transaction_ids,
                 update transact set d=:d where id=:id"""),
                                         d=completion_date,
                                         id=transaction_id).rowcount:
-                    logging.error(
+                    logger.error(
                         'Transaction {} not found'.format(transaction_id))
                     raise ResourceNotFound
             finally:
@@ -1676,7 +1686,7 @@ def transaction_delete(transaction_ids):
     """
     Delete (mark deleted) transaction
     """
-    logging.warning('Deleting transaction {}'.format(transaction_ids))
+    logger.warning('Deleting transaction {}'.format(transaction_ids))
     ids = transaction_ids if isinstance(transaction_ids,
                                         (list, tuple)) else [transaction_ids]
     for transaction_id in ids:
@@ -1687,7 +1697,7 @@ def transaction_delete(transaction_ids):
             service IS null"""),
                                 ts=parse_date(return_timestamp=False),
                                 id=transaction_id).rowcount:
-            logging.error('Transaction {} not found'.format(transaction_id))
+            logger.error('Transaction {} not found'.format(transaction_id))
             raise ResourceNotFound
         chid = tinfo.get('chain_transact_id')
         if chid:
@@ -1706,7 +1716,7 @@ def transaction_purge(_lock=True):
     try:
         db = get_db()
         dbt = db.begin()
-        logging.info('Purging deleted transactions')
+        logger.info('Purging deleted transactions')
         try:
             db.execute(
                 sql("""delete from transact where
@@ -1913,12 +1923,27 @@ def purge():
     """
     Purge deleted resources
     """
-    logging.info('Purge requested')
+    logger.info('Purge requested')
     with lock_purge:
         result = {'transaction': transaction_purge(_lock=False)}
         return result
 
 
+@core_method
+def cleanup():
+    """
+    Cleanup database
+    """
+    # cleanup archived transactions
+    logger.info('Cleanup requested')
+    get_db().execute(
+        sql("""
+        DELETE FROM transact WHERE
+            account_debit_id IS NULL AND
+            account_credit_id IS NULL
+    """))
+
+
 @core_method
 def account_list(asset=None,
                  tp=None,
@@ -2295,6 +2320,88 @@ def _account_summary(balance_type,
             yield row
 
 
+@core_method
+def archive_account(account,
+                    due_date=None,
+                    keep_deleted=False,
+                    lock_token=None):
+    """
+    Archive account transactions
+
+    Combines account transactions to due_date to a single service transaction.
+    After archiving process is finished, cleanup() method should be called to
+    remove archived transactions from the database.
+
+    Only completed transactions are archived.
+
+    WARNING: backing up database is always recommended before performing
+    archiving procedure
+
+    Args:
+        account: account to archive
+        due_date: archivation date (default: now)
+        keep_deleted: keep deleted transactions (default: False)
+    """
+    account = account.upper()
+    due_date = parse_date(due_date, return_timestamp=False)
+    db = get_db()
+    token = account_lock(account, lock_token)
+    try:
+        balance = account_balance(account, date=due_date)
+        logger.info('Archiving account transactions for {}'.format(account))
+        if balance:
+            dbt = db.begin()
+            d = datetime.datetime.now()
+            account_id = db.execute(sql("""
+                    SELECT id FROM account WHERE code=:account
+                """),
+                                    account=account).fetchone().id
+            acc_info = account_info(account)
+            if balance > 0:
+                acc_tpt = 'credit' if acc_info['passive'] else 'debit'
+            else:
+                acc_tpt = 'debit' if acc_info['passive'] else 'credit'
+            db.execute(sql("""
+            INSERT INTO transact(
+                account_{}_id, tag, note, amount, d, d_created, service)
+            VALUES (:account_id, :tag, :note, :amount, :d, :d, :s)
+            """.format(acc_tpt)),
+                       account_id=account_id,
+                       tag='archive',
+                       note='acrhived {}'.format(d.strftime('%Y-%m-%d %T')),
+                       amount=_multiply(abs(balance)),
+                       d=d,
+                       d_dcreated=d,
+                       s=True)
+            db.execute(sql("""
+                        UPDATE transact SET
+                        account_debit_id=NULL
+                        WHERE
+                            service is NULL
+                            AND d_created <= :d
+                            AND d IS NOT NULL
+                            AND account_debit_id=:account_id
+                            {}
+                    """.format(' AND deleted IS NULL' if keep_deleted else '')),
+                       d=due_date,
+                       account_id=account_id)
+            db.execute(sql("""
+                        UPDATE transact SET
+                        account_credit_id=NULL
+                        WHERE
+                            service IS NULL
+                            AND d_created <= :d
+                            AND d IS NOT NULL
+                            AND account_credit_id=:account_id
+                            {}
+                    """.format(' AND deleted IS NULL' if keep_deleted else '')),
+                       d=due_date,
+                       account_id=account_id)
+            dbt.commit()
+    finally:
+        account_unlock(account, token)
+
+
 @core_method
 def account_balance(account=None,
                     tp=None,
